////////////////////////////////////////////////////
// temp_controller_v15.c
// Copyright 2015-2016 by Jaakko Kairus
// jaakko@kair.us
//
// You may freely use and modify this software for personal use.
// Commercial use of this software or parts of it is prohibited
// without prior written permission from author.
// DISCLAIMER: This software comes with ABSOLUTELY NO WARRANTY
// 
// Ohjelman kytt ja muokkaus henkilkohtaiseen kyttn sallittu.
// Ohjelman tai sen osien kytt kaupallisiin tarkoituksiin sallittu vain tekijn luvalla.
// VASTUUVAPAUSLAUSEKE: Tekij ei ota vastuuta jos homehdutat, poltat tai
// muuten tuhoat talosi tai jos halkaiset vesijohdot tai saat shkiskun tmn ohjelman
// avulla. Eik mistn muustakaan.
//
// For more information about this project see project web page at
// http://kair.us/projects/temp_controller/
//
// Compiled with CCS compiler version 5.054
// Total compiled length 4586 bytes for CC display, 4592 for CA display
////////////////////////////////////////////////////
// Version history
// v0.1 30.4.2015
//      Initial version. int16 lookup table
// v0.2 2.5.2015
//      Changed to int32 lookup table.
//      Added 0.025 degree hysteresis to temp readout
//      Saving of parameters too EEPROM
// v0.3 10.5.2015
//      Better hysteresis, zeroes hyst if not close to flicker
//      Simplified send_saa1064_number function, now using pointer
//      to display array as argument
//      Fixed flickering leading zero in case of number smaller than 10
// v0.4 15.5.2015
//      Added compile option to choose either CA or CC display type
//      Fixed relay drive affected by digit common drive
//      Changed to use fast_io on all ports
// v1.0 4.7.2015
//		Cleanup for release
// v1.1 21.7.2015
//		Rewrote encoder readout. Simpler and works better with different
//		brands of encoders. Alps and Bourns encoders have detents at slightly
//		different position, also mechanical detents vary somewhat regarding
//		A and B signals depending on encoder position with both brands. This
//		caused the old readout code to require different settings for
//		Alps and Bourns to eliminate chattering at some detents
// v1.2 7.11.2015
// 		Now also seconds are zeroed when hour counters are reset
//		Added new setting 'low limit type'. Allows changing the behavior of
//		low temperature limit. Default setting is 'antifreeze'. With this
//		setting the _indoor_ temperature is not allowed to go below low limit.
//		With 'heater off' setting the heater is turned completely off if
//		_outdoor_ temperature is below the low limit. This can be used to save
//		electricity because mold does not grow below 0 degrees celsius.
//		Use this setting with care and make your own decision wheter it is
//		clever to use it or not..
// v1.3 7.1.2016
// 		Fixed incorrect hysteresis signs from low limit in antifreeze mode in
//		controlHeater() function
//		Now displays FW version when power is switched on
//		7 seg character defines moved to separate file 7seg_chars.h (same file
//		as used in weller driver project)
// v1.4 4.2.2016
// 		Better button debounce
// v1.5 29.5.2016
// 		Add support for optional encoder location on pins RC3, RC4, RC5
//		Fixed heater being switched on when fw version is shown
//		Increased max lowlimit and highlimit values to suit special applications

#define FW_VERSION 15
#define CC_DISPLAY

#include <16F1789.h>
#device ADC=12;
#include "7seg_chars.h"
#include <stdlib.h>
#include <stdio.h>
#include <internal_eeprom.c>	// for reading / writing int16
#fuses NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV25,NOPROTECT,NOLVP,NODEBUG,PUT,NOMCLR
// #fuses NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV25,NOPROTECT,NOLVP,NODEBUG,PUT,MCLR
#use delay(int=16MHz, clock=32MHz)
#use FAST_IO(a)
#use FAST_IO(b)
#use FAST_IO(c)
#use FAST_IO(d)
#use FAST_IO(e)
// #define LATC 0xF8B		// PIC18
#define LATC 0x10E	// PIC16 enhanced


#ifdef CC_DISPLAY
#define COMMON0 output_e(0b11111111); output_c((*LATC&0b00000100)|0b00111011);
#define COMMON1 output_c((*LATC&0b00000100)|0b11111000);
#define COMMON2 output_c((*LATC&0b00000100)|0b11111011); output_a(0b11111100);
#define COMMON3 output_a(0b11110011);
#define COMMON4 output_a(0b11111111); output_e(0b11111001);
#define COMMON5 output_e(0b11111111); output_a(0b00111111);
#define COMMON6 output_a(0b11001111);
#define COMMON7 output_a(0b11111111); output_b(0b11001111);
#define COMMON8 output_b(0b11111111); output_e(0b11111110);
#else
#define COMMON0 output_e(0b00000000); output_c((*LATC&0b00000100)+0b11000000);
#define COMMON1 output_c((*LATC&0b00000100)+0b00000011);
#define COMMON2 output_c((*LATC&0b00000100)+0b00000000); output_a(0b00000011);
#define COMMON3 output_a(0b00001100);
#define COMMON4 output_a(0b00000000); output_e(0b00000110);
#define COMMON5 output_e(0b00000000); output_a(0b11000000);
#define COMMON6 output_a(0b00110000);
#define COMMON7 output_a(0b00000000); output_b(0b00110000);
#define COMMON8 output_b(0b00000000); output_e(0b00000001);
#endif


// Messages
byte const blank[4] = {_SPACE,_SPACE,_SPACE,_SPACE}; // Blank display
byte const  diff[4] = {_D,_I,_F,_F}; //  diFF
byte const  hyst[4] = {_H,_Y,_S,_T}; //  hySt
byte const  loli[4] = {_L,_O,_L,_I}; //  LoLi
byte const  hili[4] = {_H,_I,_L,_I}; //  hiLi
byte const  bacc[4] = {_B,_A,_C,_C}; //  bAcc
byte const   set[4] = {_SPACE,_S,_E,_T}; //   SEt
byte const  to_h[4] = {_T,_O,_SPACE,_H}; //   to h
byte const  he_h[4] = {_H,_E,_SPACE,_H}; //   hE h
byte const  rese[4] = {_R,_E,_S,_E}; //   rESE
byte const  llty[4] = {_L,_L,_T,_Y}; //   LLty
byte const  anfr[4] = {_A,_N,_F,_R}; //   AnFr
byte const  hoff[4] = {_H,_O,_F,_F}; //   hoFF
byte const  vers[4] = {_V,_E,_R,_S}; //   vErS


int8 const hex_table[16]= {_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F};

typedef enum {
        ST_MAIN,
        ST_MENU_MAIN,
        ST_MENU_DIFF,
        ST_ADJ_DIFF,
        ST_MENU_HYST,
        ST_ADJ_HYST,
        ST_MENU_LOWLIMIT,
        ST_ADJ_LOWLIMIT,
        ST_MENU_HIGHLIMIT,
        ST_ADJ_HIGHLIMIT,
        ST_MENU_LOWLIMITTYPE,
        ST_ADJ_LOWLIMITTYPE,
        ST_MENU_HOURS,
        ST_MENU_HEATERHOURS,
        ST_MENU_RESET
} STATES;
STATES state = ST_MAIN;

typedef enum {
        EVT_LEFT,
        EVT_RIGHT,
        EVT_BUTTON,
        EVT_NONE
} EVENTS;
EVENTS event = EVT_NONE;

unsigned int32 ROM lookup[146] = {2024131,2020502,2016523,2012178,2007530,2002472,1997097,1991289,1985184,1978669,1971724,1964505,1956780,1948734,1940297,1931500,1922389,1912706,1902791,1892552,1881905,1871141,1860025,1848556,1836733,1824511,1811980,1799007,1785678,1772012,1757917,1743381,1728519,1713199,1697542,1681463,1665017,1648083,1630807,1613157,1595081,1576120,1556927,1537165,1517187,1496937,1476142,1455232,1433916,1412479,1390564,1368468,1346292,1323866,1301269,1278592,1255936,1232704,1209656,1186930,1163840,1140422,1117632,1094210,1071650,1048576,1025810,1003176,980695,958386,936195,914325,892585,871177,849961,829071,808421,788068,767990,748328,728879,709871,691083,672747,654720,637045,619764,602815,586236,570063,554221,538742,523659,508882,494560,480476,466777,453487,440503,427977,415664,403715,392012,380707,369821,359085,348651,338681,328896,319302,310060,301184,292529,284105,275917,267975,260284,252853,245526,238634,231856,225196,218825,212580,206634,200821,195318,189870,184634,179545,174623,
	169835,165183,160670,156279,152031,147890,143878,139977,136189,132496,129067,125700,122414,119192,116053};

unsigned int8 DISPLAY_IN[4];			// Display memory for inside temperature
unsigned int8 DISPLAY_OUT[4];			// Display memory for outside temperature
unsigned int8 COLONS;					// Display memory for clock colons and degree sign
unsigned int8 CURRENT_DIGIT = 0;

unsigned int8 oldenc = 0;
unsigned int8 oldenc_c = 0;

signed int32 temp1=0;
signed int32 temp2=0;
unsigned int32 ylempi=0;
unsigned int32 alempi=0;
signed int32 inhyst=0;
signed int32 outhyst=0;
unsigned int16 adc_tulos = 0;
unsigned int16 adc_sum = 0;
signed int16 in_buf=0;
signed int16 out_buf=0;
signed int16 new_temp=0;
signed int16 difference;
signed int16 hysteresis;
signed int16 lowlimit;
signed int16 highlimit;
int8 i=0;
int16 j=0;
unsigned int16 buttimer=0;
unsigned int8 heaterStatus=0xff;	// 0xff = Heater OFF
unsigned int16 milliseconds=0;
unsigned int16 seconds=0;
unsigned int16 heaterseconds=0;
signed int16 hours=0;
signed int16 heaterhours=0;
unsigned int8 antifreeze=1;

void saveParms();

#int_timer2
void timer2isr(void) {		// Keskeytys kest 5..10 s
	output_d(_SPACE);		// Blank current segments to prevent ghosting during common change
	switch(CURRENT_DIGIT) {
		case 0:	COMMON0
				output_d(DISPLAY_IN[0]);
//				CURRENT_DIGIT++;
				break;
		case 1:	COMMON1
				output_d(DISPLAY_IN[1]);
//				CURRENT_DIGIT++;
				break;
		case 2:	COMMON2
				output_d(DISPLAY_IN[2]);
//				CURRENT_DIGIT++;
				break;
		case 3:	COMMON3
				output_d(DISPLAY_IN[3]);
//				CURRENT_DIGIT++;
				break;
		case 4:	COMMON4
				output_d(DISPLAY_OUT[0]);
//				CURRENT_DIGIT++;
				break;
		case 5:	COMMON5
				output_d(DISPLAY_OUT[1]);
//				CURRENT_DIGIT++;
				break;
		case 6:	COMMON6
 				output_d(DISPLAY_OUT[2]);
//				CURRENT_DIGIT++;
				break;
		case 7:	COMMON7
				output_d(DISPLAY_OUT[3] _ADD_HEATER_STATUS);	// indicate heaterStatus with decimal point of digit 8
//				CURRENT_DIGIT++;
				break;
		case 8: COMMON8
				output_d(COLONS);
				CURRENT_DIGIT=255;	// overflows to 0
				break;
	}
	CURRENT_DIGIT++;
	
	// ENCODER READOUT
	if (!input(PIN_B7))		// encoder connected to icsp port
	{
		if (!input(PIN_B6) && (oldenc & 0x40))
			event = EVT_RIGHT;
		if (input(PIN_B6) && !(oldenc & 0x40))
			event = EVT_LEFT;
	}	
	else if (!input(PIN_C3))	// encoder connected to port C pins
	{
		if (!input(PIN_C5) && (oldenc_c & 0x20))
			event = EVT_RIGHT;
		if (input(PIN_C5) && !(oldenc_c & 0x20))
			event = EVT_LEFT;
	}	
	oldenc = input_b();
	oldenc_c = input_c();

	if (!input(PIN_E3)||!input(PIN_C4)||(buttimer>1&&buttimer<20))	// 20 ms debounce
		buttimer++;
	else
		buttimer=0;
		
	if (buttimer==1)
		event = EVT_BUTTON;
	//	else if (buttimer>1000)
	//		event = EVT_LONG_PRESS;
	// End reading encoder
	
	milliseconds++;
	if (milliseconds>999)
	{
		milliseconds=0;
		seconds++;
		if (heaterStatus==0x7f)
			heaterseconds++;
	}

	if (seconds>3599)
	{
		seconds=0;
		hours++;
		write_int16_eeprom(8, hours);
	}	

	if (heaterseconds>3599)
	{
		heaterseconds=0;
		heaterhours++;
		write_int16_eeprom(10, heaterhours);
	}	

}

void send_saa1064_num(int8* DISPLAY, int8 mode, signed int16 num)
{
   signed int8 j;                       // Digit index
   signed int16 temp16;
   temp16 = labs(num); 
   j = 3;
   
   while (j >= 0)
   {
 	   if (j<3 && temp16 < 10) break;	// prevent leading zeroes in the first place
	   if (j != 3) temp16 /= 10;
	   if (j==2&&mode==1)	/* If mode==1 (temperature), show decimal point) */
	   		DISPLAY[j] = hex_table[temp16 %10] _ADDPOINT;
	   else
	   		DISPLAY[j] = hex_table[temp16 %10];
       j--;
    }

   // Do leading zero blanking
	if (num<1000 && num>-100) DISPLAY[0] = _SPACE;
	if (num<100 && num>-10) DISPLAY[1] = _SPACE;
   	if (num<0&&labs(num)>99) DISPLAY[0] = _MINUS;
   	if (num<0&&labs(num)<100) DISPLAY[1] = _MINUS;
 	if (labs(num)<10 && mode==0)  DISPLAY[2] = _SPACE;
 	if (labs(num)<10 && mode==1)  DISPLAY[2] = _0P;
   	
}

void readAdc(int16* adc_r, unsigned int8 channel) {
	set_adc_channel(channel);
	i=0;
	adc_sum = 0;
	while (i<16) { // sample 16 times to get a virtual 16 bit reading..
		delay_us(10);
		read_adc(ADC_START_ONLY);
		while(!(adc_done())) {    
		} 
		adc_tulos = read_adc(ADC_READ_ONLY);
		adc_sum = adc_sum + adc_tulos;
		i++;
	}
	*adc_r = adc_sum;
}

void lookup_32bit(unsigned int32* adc_sum, signed int16* temperature, signed int32* temphyst)
{
	for (j=0;j<146;j++)
	{
		if (*adc_sum>lookup[j])
			break;
	}
	
	ylempi = lookup[j];
	alempi = lookup[j-1];
	temp1=10*(alempi-*adc_sum);
	temp2=(alempi-ylempi);
//	new_temp = temp1/temp2+ 10*(j-41); // -41 because our lookup table begins from -40
	new_temp = (temp1+*temphyst)/temp2+ 10*(j-41); // -41 because our lookup table begins from -40
	
	/* Add hysteresis of 0.025 degree to avoid flickering between two values
	   this takes up 400 bytes of code(!)  */
	if (new_temp > *temperature)
	{
		*temphyst = temp2/4;
		*temperature = new_temp;
	}	
	if (new_temp < *temperature)
	{
		*temphyst = -temp2/4;
		*temperature = new_temp;
	}	

	new_temp = temp1%temp2;
	temp2=temp2/3;
	temp1=2*temp2;
	if (temp2 < new_temp && new_temp < temp1)	// zero hysteresis if not close to flicker
		*temphyst = 0;
		
}

void menuMachine()
{
		/* Switch the state and the event to execute the right transition. */
	switch(state)
	{
		case ST_MAIN:
			send_saa1064_num(DISPLAY_IN,1,in_buf);
			send_saa1064_num(DISPLAY_OUT,1,out_buf);

			switch(event)
			{
				case EVT_BUTTON:
					state = ST_MENU_MAIN;
					break;
			}
			break;
		case ST_MENU_MAIN:
			memcpy (DISPLAY_IN, bacc, sizeof (DISPLAY_IN));
			memcpy (DISPLAY_OUT, blank, sizeof (DISPLAY_OUT));
			switch(event)
			{
				case EVT_BUTTON:
					state = ST_MAIN;
					break;
				case EVT_LEFT:
					memcpy (DISPLAY_OUT, blank, sizeof (DISPLAY_IN));
					state = ST_MENU_RESET;
					break;
				case EVT_RIGHT:
					state = ST_MENU_DIFF;
					break;
			}
			break;
		case ST_MENU_DIFF:
			memcpy (DISPLAY_IN, diff, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, difference);
			switch(event)
			{
				case EVT_BUTTON:
					state = ST_ADJ_DIFF;
					break;
				case EVT_LEFT:
					state = ST_MENU_MAIN;
					break;
				case EVT_RIGHT:
					state = ST_MENU_HYST;
					break;
			}
			break;
		case ST_ADJ_DIFF:
			memcpy (DISPLAY_IN, set, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, difference);
			switch(event)
			{
				case EVT_BUTTON:
					saveParms();
					state = ST_MENU_DIFF;
					break;
				case EVT_LEFT:
					if(difference>0)
						difference--;
					break;
				case EVT_RIGHT:
					if(difference<200)
						difference++;
					break;
			}
			break;
		case ST_MENU_HYST:
			memcpy (DISPLAY_IN, hyst, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, hysteresis);
			switch(event)
			{
				case EVT_BUTTON:
					state = ST_ADJ_HYST;
					break;
				case EVT_LEFT:
					state = ST_MENU_DIFF;
					break;
				case EVT_RIGHT:
					state = ST_MENU_LOWLIMIT;
					break;
			}
			break;
		case ST_ADJ_HYST:
			memcpy (DISPLAY_IN, set, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, hysteresis);
			switch(event)
			{
				case EVT_BUTTON:
					saveParms();
					state = ST_MENU_HYST;
					break;
				case EVT_LEFT:
					if(hysteresis>0)
						hysteresis--;
					break;
				case EVT_RIGHT:
					if(hysteresis<50)
						hysteresis++;
					break;
			}
			break;
		case ST_MENU_LOWLIMIT:
			memcpy (DISPLAY_IN, loli, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, lowlimit);
			switch(event)
			{
				case EVT_BUTTON:
					state = ST_ADJ_LOWLIMIT;
					break;
				case EVT_LEFT:
					state = ST_MENU_HYST;
					break;
				case EVT_RIGHT:
					state = ST_MENU_HIGHLIMIT;
					break;
			}
			break;
		case ST_ADJ_LOWLIMIT:
			memcpy (DISPLAY_IN, set, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, lowlimit);
			switch(event)
			{
				case EVT_BUTTON:
					saveParms();
					state = ST_MENU_LOWLIMIT;
					break;
				case EVT_LEFT:
					if(lowlimit>-200)
						lowlimit--;
					break;
				case EVT_RIGHT:
					if(lowlimit<500)
						lowlimit++;
					break;
			}
			break;
		case ST_MENU_HIGHLIMIT:
			memcpy (DISPLAY_IN, hili, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, highlimit);
			switch(event)
			{
				case EVT_BUTTON:
					state = ST_ADJ_HIGHLIMIT;
					break;
				case EVT_LEFT:
					state = ST_MENU_LOWLIMIT;
					break;
				case EVT_RIGHT:
					state = ST_MENU_LOWLIMITTYPE;
					break;
			}
			break;
		case ST_ADJ_HIGHLIMIT:
			memcpy (DISPLAY_IN, set, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,1, highlimit);
			switch(event)
			{
				case EVT_BUTTON:
					saveParms();
					state = ST_MENU_HIGHLIMIT;
					break;
				case EVT_LEFT:
					if(highlimit>-200)
						highlimit--;
					break;
				case EVT_RIGHT:
					if(highlimit<600)
						highlimit++;
					break;
			}
			break;
		case ST_MENU_LOWLIMITTYPE:
			memcpy (DISPLAY_IN, llty, sizeof (DISPLAY_IN));
			if (antifreeze)
				memcpy (DISPLAY_OUT, anfr, sizeof (DISPLAY_OUT));
			else
				memcpy (DISPLAY_OUT, hoff, sizeof (DISPLAY_OUT));
			switch(event)
			{
				case EVT_BUTTON:
					state = ST_ADJ_LOWLIMITTYPE;
					break;
				case EVT_LEFT:
					state = ST_MENU_HIGHLIMIT;
					break;
				case EVT_RIGHT:
					state = ST_MENU_HOURS;
					break;
			}
			break;
		case ST_ADJ_LOWLIMITTYPE:
			memcpy (DISPLAY_IN, set, sizeof (DISPLAY_IN));
			if (antifreeze)
				memcpy (DISPLAY_OUT, anfr, sizeof (DISPLAY_OUT));
			else
				memcpy (DISPLAY_OUT, hoff, sizeof (DISPLAY_OUT));
			switch(event)
			{
				case EVT_BUTTON:
					saveParms();
					state = ST_MENU_LOWLIMITTYPE;
					break;
				case EVT_LEFT:
				case EVT_RIGHT:
					if(antifreeze)
						antifreeze=0;
					else
						antifreeze=1;
					break;
			}
			break;
		case ST_MENU_HOURS:
			memcpy (DISPLAY_IN, to_h, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,0, hours);
			switch(event)
			{
				case EVT_LEFT:
					state = ST_MENU_LOWLIMITTYPE;
					break;
				case EVT_RIGHT:
					state = ST_MENU_HEATERHOURS;
					break;
			}
			break;
		case ST_MENU_HEATERHOURS:
			memcpy (DISPLAY_IN, he_h, sizeof (DISPLAY_IN));
			send_saa1064_num(DISPLAY_OUT,0, heaterhours);
			switch(event)
			{
				case EVT_LEFT:
					state = ST_MENU_HOURS;
					break;
				case EVT_RIGHT:
					state = ST_MENU_RESET;
					memcpy (DISPLAY_OUT, blank, sizeof (DISPLAY_IN));
					break;
			}
			break;
		case ST_MENU_RESET:
			memcpy (DISPLAY_IN, rese, sizeof (DISPLAY_IN));
			switch(event)
			{
				case EVT_BUTTON:
					seconds = 0;
					heaterseconds = 0;
					hours = 0;
					heaterhours = 0;
					write_int16_eeprom(8, hours);
					write_int16_eeprom(10, heaterhours);
					memcpy (DISPLAY_OUT, rese, sizeof (DISPLAY_IN));
					break;
				case EVT_LEFT:
					state = ST_MENU_HEATERHOURS;
					break;
				case EVT_RIGHT:
					state = ST_MENU_MAIN;
					break;
			}
			break;
	} // end menu case
	event = EVT_NONE;
}		

void controlHeater()
{
	if (antifreeze) // Low limit type is antifreeze
	{
		if (heaterStatus==0x7f)	// heater is on
		{
			if (in_buf-out_buf-hysteresis/2>difference && in_buf-hysteresis/2>lowlimit || in_buf-hysteresis/2>highlimit)
				heaterStatus = 0xff;
		}
		else					// heater is off
		{
			if (in_buf-out_buf+hysteresis/2<difference && in_buf+hysteresis/2<highlimit || in_buf+hysteresis/2<lowlimit)
				heaterStatus = 0x7f;
		}
	}	
	else	// Low limit type is heater off to save power
	{
		if (heaterStatus==0x7f)
		{
			if (in_buf-out_buf-hysteresis/2>difference || out_buf+hysteresis/2<lowlimit || in_buf-hysteresis/2>highlimit)
				heaterStatus = 0xff;
		}
		else
		{
			if (in_buf-out_buf+hysteresis/2<difference && in_buf+hysteresis/2<highlimit && out_buf-hysteresis/2>lowlimit)
				heaterStatus = 0x7f;
		}
	}	
}	

void saveParms()
{
	write_int16_eeprom(0, difference);
	write_int16_eeprom(2, hysteresis);
	write_int16_eeprom(4, lowlimit);
	write_int16_eeprom(6, highlimit);
	write_eeprom(12, antifreeze);
}	

void main()
{
	int8 kierrokset=1;	// start from 1 to prevent heater 'blinking' during round 0. fix with something more clever
	unsigned int16 temp_reading=0;
	unsigned int32 in_sum=0;
	unsigned int32 out_sum=0;
	// INITIALIZE TIMER 2 FOR 1MS PERIOD
	setup_timer_2(T2_DIV_BY_16,250,2); // set timer2 to 1ms (prescaler 16 and postscaler 2, period 250)
	enable_interrupts(INT_TIMER2);
	enable_interrupts(GLOBAL);
	set_tris_a(0x00);
	set_tris_b(0b11001111);
	set_tris_c(0b00111000);
	set_tris_d(0x00);
	set_tris_e(0x11111000);
	port_b_pullups(0b11000000);
	port_c_pullups(0b00111000);	// pullups for optional encoder location
//	port_e_pullups(0b00001000);	// MCLR pullup


	output_low(PIN_C2);	// keep heater off during fw version is shown

	COLONS=_SPACE;
		
	setup_adc_ports( sAN8|sAN9|VSS_VDD);
	setup_adc(ADC_CLOCK_DIV_64);
	
	// DISPLAY FW VERSION AT BOOT //
	memcpy (DISPLAY_IN, vers, sizeof (DISPLAY_IN));
	send_saa1064_num(DISPLAY_OUT,1, FW_VERSION);
	delay_ms(1800);

	if (read_eeprom(0)==0xFF && read_eeprom(1)==0xFF)	// Reprogramming has erased EEPROM or first time run
	{
		write_int16_eeprom(0, 50);					// Set difference to 5 degrees
		write_int16_eeprom(2, 10);					// Set hysteresis 1 degree
		write_int16_eeprom(4, -50);					// Set low limit to -5 degrees
		write_int16_eeprom(6, 210);					// Set high limit to 21
		write_int16_eeprom(8, hours);				// Zero hours
		write_int16_eeprom(10, heaterhours);		// Zero heaterhours
		write_eeprom(12, antifreeze);				// Set antifreeze to 1
	}	
	difference = read_int16_eeprom(0);
	hysteresis = read_int16_eeprom(2);			// must use even addresses because one int16 takes two bytes
	lowlimit = read_int16_eeprom(4);
	highlimit = read_int16_eeprom(6);
	hours = read_int16_eeprom(8);
	heaterhours = read_int16_eeprom(10);
	antifreeze = read_eeprom(12);


	/* Infinite loop */
	while(1)
	{
		menuMachine();			// menu machine kest 7 s staattinen teksti, 0.1...0.3ms yksi lukema, 0.3...0.6ms kaksi lukemaa
								// lukemassa 0.1...0.3 haarukka riippuu siit montako desimaalia nkyy
		readAdc(&temp_reading, 8);	// read temperature ja in_sum summaus vie n. 0.7 ms (10 us viive ja ADC_CLOCK_DIV64 @ 32 MHz)
		in_sum=in_sum+temp_reading;			// Kutsutaan menuMachniea adc lukujen vliss ettei missata encoder eventtej
		
		menuMachine();
		readAdc(&temp_reading, 9);
		out_sum=out_sum+temp_reading;

		if (heaterStatus==0x7f)
			output_high(PIN_C2);
		else
			output_low(PIN_C2);
		
		kierrokset++;
		if (kierrokset==1)
			controlHeater();
		
		if (kierrokset==32)			
		{
			menuMachine();
			lookup_32bit(&in_sum, &in_buf, &inhyst);	// lookup32 kest n 1.7 ms mutta vlill vhemmn?
			in_sum=0;
			menuMachine();
			lookup_32bit(&out_sum, &out_buf, &outhyst);
			out_sum=0;
			kierrokset=0;
		}
	
	}	// end while
}